var expect = require('chai').expect;
var Database = require('../.');
var db;

before(function () {
	db = new Database('temp/' + require('path').basename(__filename).split('.')[0] + '.db');
});

describe('Database#exec()', function () {
	it('should throw an exception if a string is not provided', function () {
		expect(function () {db.exec(123);}).to.throw(TypeError);
		expect(function () {db.exec(0);}).to.throw(TypeError);
		expect(function () {db.exec(null);}).to.throw(TypeError);
		expect(function () {db.exec();}).to.throw(TypeError);
		expect(function () {db.exec(new String('CREATE TABLE entries (a TEXT, b INTEGER)'));}).to.throw(TypeError);
	});
	it('should throw an exception if invalid SQL is provided', function () {
		expect(function () {db.exec('CREATE TABLE entries (a TEXT, b INTEGER');}).to.throw(Error);
	});
	it('should throw an exception if used on a readonly database connection', function () {
		var readonly = new Database(db.name, {readonly: true});
		expect(function () {readonly.exec('CREATE TABLE entries (a TEXT, b INTEGER)');}).to.throw(TypeError);
	});
	it('should execute the SQL, returning the database object itself', function () {
		var returnValues = [];
		
		var r1 = db.exec('CREATE TABLE entries (a TEXT, b INTEGER)');
		var r2 = db.exec('INSERT INTO entries VALUES (\'foobar\', 44); INSERT INTO entries VALUES (\'baz\', NULL);');
		var r3 = db.exec('SELECT * FROM entries');
		
		expect(r1).to.equal(db);
		expect(r2).to.equal(db);
		expect(r3).to.equal(db);
		
		var rows = db.prepare('SELECT * FROM entries').all();
		expect(rows.length).to.equal(2);
		expect(rows[0].a).to.equal('foobar');
		expect(rows[0].b).to.equal(44);
		expect(rows[1].a).to.equal('baz');
		expect(rows[1].b).to.equal(null);
	});
});
